import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Button, Modal, notification, Drawer } from 'antd'

import asyncSpinComponent from '@/utils/asyncSpinComponent'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import MKEmitter from '@/utils/events.js'
import MkIcon from '@/components/mk-icon'
import './index.scss'

const SubTabTable = asyncSpinComponent(() => import('@/tabviews/subtabtable'))
const CustomPage = asyncSpinComponent(() => import('@/tabviews/custom'))

class PopupButton extends Component {
  static propTpyes = {
    show: PropTypes.any,              // 按钮显示样式控制
    BID: PropTypes.string,            // 主表ID
    BData: PropTypes.any,             // 主表数据
    selectedData: PropTypes.any,      // 子表中选择数据
    Tab: PropTypes.any,               // 如果当前元素为标签时，tab为标签信息
    MenuID: PropTypes.string,         // 菜单ID
    btn: PropTypes.object,            // 按钮
    setting: PropTypes.any,           // 页面通用设置
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    visible: false,
    popData: null,
    primaryId: '',
    disabled: false,
    hidden: false,
    loading: false,
  }

  UNSAFE_componentWillMount () {
    const { btn, selectedData } = this.props
    let disabled = false

    if (btn.controlField && selectedData && selectedData.length > 0) { // 表格中按钮隐藏控制
      selectedData.forEach(item => {
        let s = item[btn.controlField] + ''
        if (s === btn.controlVal || (btn.controlVal && btn.controlVal.split(',').includes(s))) {
          disabled = true
        }
      })
      this.setState({disabled, hidden: disabled && btn.control === 'hidden'})
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('popclose', this.syspopclose)
    MKEmitter.addListener('triggerBtnId', this.actionTrigger)
    MKEmitter.addListener('openNewTab', this.openNewTab)
    MKEmitter.addListener('refreshPopButton', this.refreshPopButton)
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { btn, selectedData } = this.props

    if (btn.controlField && !is(fromJS(nextProps.selectedData || []), fromJS(selectedData || []))) {
      let disabled = false

      if (nextProps.selectedData && nextProps.selectedData.length > 0) { // 表格中按钮隐藏控制
        nextProps.selectedData.forEach(item => {
          let s = item[btn.controlField] + ''
          if (s === btn.controlVal || (btn.controlVal && btn.controlVal.split(',').includes(s))) {
            disabled = true
          }
        })
      }
      this.setState({disabled, hidden: disabled && btn.control === 'hidden'})
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('popclose', this.syspopclose)
    MKEmitter.removeListener('openNewTab', this.openNewTab)
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
    MKEmitter.removeListener('refreshPopButton', this.refreshPopButton)
  }
  
  /**
   * @description 打开新标签时，关闭弹窗标签页
   */
  openNewTab = () => {
    this.setState({
      visible: false,
      loading: false
    })
  }

  /**
   * @description 弹窗标签页刷新
   */
  refreshPopButton = (tabId) => {
    const { btn } = this.props

    if (btn.uuid !== tabId) return

    if (btn.$view === 'CustomPage') {
      if (btn.popClose !== 'never') {
        MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.popClose, btn)
      }
    } else {
      let position = btn.popClose !== 'never' ? btn.popClose : 'grid'
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, position, btn)
    }
  }

  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId, record, type) => {
    const { setting, Tab, BID, btn, selectedData } = this.props
    const { loading, disabled } = this.state

    if ((triggerId && btn.uuid !== triggerId) || loading || disabled) return

    if (((Tab && Tab.supMenu) || setting.supModule) && !BID) {
      notification.warning({
        top: 92,
        message: '需要上级主键值！',
        duration: 5
      })
      return
    } else if (type === 'linkbtn' && selectedData && selectedData.length === 1) {
      if (record[0].$Index !== selectedData[0].$Index) {
        return
      }
    }

    let data = record || selectedData || []

    if (btn.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: this.state.dict['main.action.confirm.selectSingleLine'],
        duration: 5
      })
      return
    } else if (setting.tabType === 'subtab') {
      notification.warning({
        top: 92,
        message: '弹窗页面不支持此设置！',
        duration: 5
      })
      return
    }

    let _data = null
    let primaryId = ''

    if (btn.Ot === 'requiredSgl' && setting.primaryKey) {
      _data = data[0]
      primaryId = _data[setting.primaryKey] || ''
    }

    this.setState({
      loading: true,
      popData: _data,
      primaryId: primaryId,
      visible: true
    })

    if (window.GLOB.systemType === 'production') {
      MKEmitter.emit('queryTrigger', {menuId: btn.uuid, name: '弹窗（标签）'})
    }
  }

  syspopclose = () => {
    if (!this.state.visible) return

    this.popclose()
  }

  /**
   * @description 弹窗关闭
   */
  popclose = () => {
    const { btn } = this.props

    this.setState({
      visible: false,
      loading: false
    })

    if (btn.popClose !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.popClose, btn)
    }
  }

  getPop = () => {
    const { btn } = this.props
    const { popData, primaryId, visible } = this.state

    let ratio = btn.ratio || 85

    if (btn.display !== 'drawer') {
      if (ratio > 100) {
        ratio = ratio + 'px'
      } else {
        ratio = ratio + 'vw'
      }
      return <Modal
        className={'popview-modal ' + (btn.$view === 'CustomPage' ? 'custom-popview' : '')}
        title={btn.label}
        width={ratio}
        maskClosable={btn.clickouter === 'close'}
        visible={visible}
        onCancel={this.popclose}
        footer={[
          <Button key="close" onClick={this.popclose}>{this.state.dict['main.close']}</Button>
        ]}
        destroyOnClose
      >
        {btn.$view !== 'CustomPage' ? <SubTabTable
          Tab={btn}
          MenuID={btn.linkTab}
          SupMenuID={this.props.MenuID}
          BID={popData ? primaryId : this.props.BID}
          BData={popData || this.props.BData}
        /> : null}
        {btn.$view === 'CustomPage' ? <CustomPage Tab={btn} MenuID={btn.uuid} param={{$BID: (popData ? primaryId : this.props.BID), ...(popData || this.props.BData || {})}} /> : null}
      </Modal>
    } else {
      let height = '100vh'
      let width = '100vw'
      if (btn.placement === 'top' || btn.placement === 'bottom') {
        if (ratio > 100) {
          ratio = ratio + 'px'
        } else {
          ratio = ratio + 'vh'
        }
        
        height = ratio
      } else {
        if (ratio > 100) {
          ratio = ratio + 'px'
        } else {
          ratio = ratio + 'vw'
        }

        width = ratio
      }

      return (
        <Drawer
          title={btn.label}
          className={btn.$view === 'CustomPage' ? 'custom-drawer-popview' : ''}
          width={width}
          height={height}
          maskClosable={btn.clickouter === 'close'}
          onClose={this.popclose}
          visible={visible}
          placement={btn.placement || 'right'}
          destroyOnClose
        >
          {btn.$view !== 'CustomPage' ? <SubTabTable
            Tab={btn}
            MenuID={btn.linkTab}
            SupMenuID={this.props.MenuID}
            BID={popData ? primaryId : this.props.BID}
            BData={popData || this.props.BData}
          /> : null}
          {btn.$view === 'CustomPage' ? <CustomPage Tab={btn} MenuName={btn.label} MenuID={btn.uuid} param={{$BID: (popData ? primaryId : this.props.BID), ...(popData || this.props.BData || {})}} /> : null}
          <div className="close-drawer">
            <Button onClick={this.popclose}>
              关闭
            </Button>
          </div>
        </Drawer>
      )
    }
  }

  render() {
    const { btn, show } = this.props
    const { loading, disabled, hidden } = this.state

    if (hidden) return null

    let label = ''
    let icon = ''

    if (show === 'button') {
      label = btn.label
      icon = btn.icon || ''
    } else if (show === 'link') {
      label = <span>{btn.label}{btn.icon ? <MkIcon style={{marginLeft: '8px'}} type={btn.icon}/> : ''}</span>
      icon = ''
    } else if (show === 'icon') {
      icon = btn.icon || ''
    } else {
      label = btn.label
    }

    return (
      <div style={{display: 'inline-block'}} onClick={(e) => e.stopPropagation()}>
        {show === 'actionList' ? <Button
          className={'mk-btn mk-' + btn.class}
          icon={btn.icon}
          disabled={disabled}
          onClick={() => {this.actionTrigger()}}
          loading={loading}
        >{btn.label}</Button> : null}
        {show !== 'actionList' ? <Button
          type="link"
          title={show === 'icon' ? btn.label : ''}
          loading={loading}
          disabled={disabled}
          style={btn.style}
          icon={icon}
          onClick={() => {this.actionTrigger()}}
        >{label}</Button> : null}
        {this.getPop()}
      </div>
    )
  }
}
export default PopupButton